@@ -278,6 +278,8 @@ aggregator_putmetric(
 			for (i = 0; i < s->bucketcnt; i++) {
 				invocation->buckets[i].start = now + (i * s->interval);
 				invocation->buckets[i].cnt = 0;
+				invocation->buckets[i].entries.size = 0;
+				invocation->buckets[i].entries.values = NULL;
 			}
 
 			invocation->next = compute->invocations_ht[omhtbucket];
@@ -320,7 +322,8 @@ aggregator_putmetric(
 		if (compute->entries_needed) {
 			if (bucket->cnt == entries->size) {
 #define E_I_SZ 64
-				double *new = realloc(entries->values, entries->size + E_I_SZ);
+				double *new = realloc(entries->values,
+						sizeof(double) * (entries->size + E_I_SZ));
 				if (new == NULL) {
 					logerr("aggregator: out of memory creating entry bucket "
 							"(%s from %s)", ometric, metric);
@@ -361,6 +364,7 @@ aggregator_expire(void *sub)
 	struct _aggr_invocations *inv;
 	struct _aggr_invocations *lastinv;
 	double *values;
+	size_t len;
 	int i;
 	unsigned char j;
 	int work;
@@ -450,7 +454,7 @@ aggregator_expire(void *sub)
 												"%s %f %lld\n",
 												inv->metric,
 												values[n - 1],
-												b->start + s->interval);
+												ts);
 									}	break;
 									case VAR:
 									case SDEV: {
@@ -465,7 +469,7 @@ aggregator_expire(void *sub)
 												inv->metric,
 												c->type == VAR ? ksum :
 													sqrt(ksum),
-												b->start + s->interval);
+												ts);
 									}	break;
 								}
 								server_send(submission, strdup(metric), 1);
@@ -475,13 +479,18 @@ aggregator_expire(void *sub)
 							/* move the bucket to the end, to make room for
 							 * new ones */
 							pthread_mutex_lock(&s->bucketlock);
-							memmove(&inv->buckets[0], &inv->buckets[1],
+							b = &inv->buckets[0];
+							len = b->entries.size;
+							values = b->entries.values;
+							memmove(b, &inv->buckets[1],
 									sizeof(*b) * (s->bucketcnt - 1));
 							b = &inv->buckets[s->bucketcnt - 1];
 							b->cnt = 0;
 							b->start =
 								inv->buckets[s->bucketcnt - 2].start +
 								s->interval;
+							b->entries.size = len;
+							b->entries.values = values;
 							pthread_mutex_unlock(&s->bucketlock);
 
 							work++;
@@ -533,6 +542,35 @@ aggregator_expire(void *sub)
 		}
 	}
 
+	/* free up value buckets */
+	for (s = aggregators; s != NULL; s = s->next) {
+		while (s->computes != NULL) {
+			c = s->computes;
+
+			free((void *)c->metric);
+			for (i = 0; i < 1 << AGGR_HT_POW_SIZE; i++) {
+				inv = c->invocations_ht[i];
+
+				while (inv != NULL) {
+					struct _aggr_invocations *invocation = inv;
+
+					free(inv->metric);
+					if (c->entries_needed)
+						for (j = 0; j < s->bucketcnt; j++)
+							if (inv->buckets[j].entries.values)
+								free(inv->buckets[j].entries.values);
+					free(inv->buckets);
+
+					inv = invocation->next;
+					free(inv);
+				}
+			}
+
+			s->computes = c->next;
+			free(c);
+		}
+	}
+
 	return NULL;
 }
 
